#ifndef FDS_IO
#define FDS_IO

#include "fds.h"

#ifndef HANDLE
typedef void *HANDLE;
#endif

class FileClass {
public:
	virtual ~FileClass()
	{
	}
	virtual char *File_Name() = 0;
	virtual char *Set_Name(const char* name) = 0;
	virtual bool Create() = 0;
	virtual bool Delete() = 0;
	virtual bool Is_Available(int handle) = 0;
	virtual bool Is_Open() = 0;
	virtual int Open(const char* name, int mode) = 0;
	virtual int Open(int mode) = 0;
	virtual int Read(void* buffer, int size) = 0;
	virtual int Seek(int offset, int origin) = 0;
	virtual int Tell()
	{
		return Seek(0,1);
	}
	virtual int Size() = 0;
	virtual int Write(void* buffer, int size) = 0;
	virtual void Close() = 0;
	virtual unsigned long Get_Date_Time()
	{
		return 0;
	}
	virtual bool Set_Date_Time()
	{
		return false;
	}
	virtual void Error(int a, int b, char *c) = 0;
	virtual HANDLE Get_File_Handle()
	{
		return (HANDLE)-1;
	}
	virtual void Bias(int start, int length) = 0;
};

class FileFactoryClass {
public:
	virtual ~FileFactoryClass();
	virtual FileClass* Get_File(const char* Filename) = 0;
	virtual void Return_File(FileClass* File) = 0;
};

struct ChunkHeader {
	unsigned long ChunkType;
	unsigned long ChunkSize;
};

struct MicroChunkHeader {
	unsigned char ChunkType;
	unsigned char ChunkSize;
};

struct IOVector2Struct {
	float X;
	float Y;
};

struct IOVector3Struct {
	float X;
	float Y;
	float Z;
};

struct IOVector4Struct {
	float X;
	float Y;
	float Z;
	float W;
};

struct IOQuaternionStruct {
	float Q[4];
};

class ChunkLoadClass {
private:
	FileClass* File;
	int StackIndex;
	unsigned long PositionStack[256];
	ChunkHeader HeaderStack[256];
	bool InMicroChunk;
	int MicroChunkPosition;
	MicroChunkHeader MCHeader;
public:
	ChunkLoadClass(FileClass *file);
	bool Open_Chunk();
	bool Peek_Next_Chunk(unsigned int *id, unsigned int *length);
	bool Close_Chunk();
	unsigned long Cur_Chunk_ID();
	unsigned long Cur_Chunk_Length();
	int Cur_Chunk_Depth();
	int Contains_Chunks();
	bool Open_Micro_Chunk();
	bool Close_Micro_Chunk();
	unsigned long Cur_Micro_Chunk_ID();
	unsigned long Cur_Micro_Chunk_Length();
	long Seek(unsigned long nbytes);
	long Read(void *buf, unsigned long nbytes);
	long Read(IOVector2Struct *v);
	long Read(IOVector3Struct *v);
	long Read(IOVector4Struct *v);
	long Read(IOQuaternionStruct *q);
};

class ChunkSaveClass {
	class FileClass* File;
	int StackIndex;
	int PositionStack[256];
	struct ChunkHeader HeaderStack[256];
	bool InMicroChunk;
	int MicroChunkPosition;
	struct MicroChunkHeader MCHeader;
public:
	ChunkSaveClass(FileClass *file);
	bool Begin_Chunk(unsigned long id);
	bool End_Chunk();
	int Cur_Chunk_Depth();
	bool Begin_Micro_Chunk(unsigned long id);
	bool End_Micro_Chunk();
	unsigned long Write(void* buf,unsigned long nbytes);
	unsigned long Write(IOVector2Struct& v);
	unsigned long Write(IOVector3Struct& v);
	unsigned long Write(IOVector4Struct& v);
	unsigned long Write(IOQuaternionStruct& q);
};

class INIClass {
public:
	int Entry_Count(const char *section);
	const char *Get_Entry(const char *section, int Num);
	int Get_Int(char const *section, char const *entry, int defaultvalue);
	float Get_Float(char const *section, char const *entry, float defaultvalue);
	bool Get_Bool(char const *section ,char const *entry, bool defaultvalue);
	const char *Get_String(char const *section, char const *entry, char const *defaultvalue, char *result, int size);
};

INIClass *Get_INI(const char *file);
void Release_INI(INIClass *ini);

#endif
